package com.qmth.cqb.paper.service.export;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.bind.JAXBElement;

import org.apache.commons.lang3.StringUtils;
import org.docx4j.XmlUtils;
import org.docx4j.jaxb.Context;
import org.docx4j.wml.Body;
import org.docx4j.wml.P;
import org.docx4j.wml.R;
import org.docx4j.wml.Text;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.core.enums.CourseLevel;
import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

import com.qmth.cqb.paper.dao.AudioTimeConfigRepo;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.service.ExamFileService;
import com.qmth.cqb.paper.service.PaperDetailService;
import com.qmth.cqb.paper.service.PaperService;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.service.QuestionAudioService;
import com.qmth.cqb.utils.BeanCopierUtil;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.exception.PaperException;
import com.qmth.cqb.utils.word.DocxProcessUtil;


/**
 * @describle 初始化导出对象service
 * @author weiwenhai 
 * @date 2017.10.17
 */
@Service("initPaperExpService")
public  class InitPaperExpService {
	
	@Autowired
	PaperRepo paperRepo;

	@Autowired
	PaperService paperService;

	@Autowired
	PaperDetailRepo paperDetailRepo;

	@Autowired
	PaperDetailUnitRepo paperDetailUnitRepo;
	
	@Autowired
	ExamFileService examFileService;

	@Autowired
	private PaperDetailService paperDetailService;
	
	@Autowired
	private QuestionAudioService questionAudioService;
	
	@Autowired
	QuesRepo quesRepo;
	
	@Autowired
	AudioTimeConfigRepo audioTimeConfigRepo;

	public static final String FILL_BLANK_QUESTION_FLAG = "###";
	
	public static final String TEMP_FILE_EXP = "docxExport/";
	
	/**
	 * Word文件后缀
	 */
	public static final String DOCX_SUFFIX = ".docx";
	/**
	 * Excel文件后缀
	 */
	public static final String EXCEL_SUFFIX = ".xlsx";
	/**
	 * ZIP文件后缀
	 */
	public static final String ZIP_SUFFIX = ".zip";
	
	public static final String ENCODING = "UTF-8";
	
	@Value("${upyun.bucketName}")
	public String bucketName;
	
	@Value("${upyun.userName}")
	public String userName;
	
	@Value("${upyun.password}")
	public String password;
	
	@Value("${upyun.uploadUrl}")
	public String uploadUrl;
	
	@Value("${upyun.downloadUrl}")
	public String downloadUrl;
	
	/**
	 * 根据paper得到导出试卷对象PaperExp
     * @param paperId
     * @return
     * @throws Exception
     */
    public PaperExp initPaperExp(String paperId) throws Exception{
    	//根据paperId取到paper对象
    	Paper paper = paperRepo.findOne(paperId);
    	paperService.formatPaper(paper, null);
    	if (paper == null) {
			throw new PaperException("该试卷不存在");
		}
    	//创建试卷导出的dto
    	PaperExp paperExpDto = BeanCopierUtil.copyProperties(paper, PaperExp.class);
    	paperExpDto.setCourseLevel(paper.getLevel() == null?CourseLevel.ALL.getName():paper.getLevel().getName());
    	//获取原试卷所有大题
    	List<PaperDetail> paperDetails = paperDetailRepo.findByPaperOrderByNumber(paper);
    	//生成新的大题对象集合，加添对应的小题
    	List<PaperDetailExp> paperDetailExps = separatePaperDetailUntils(paperDetails);
    	//给新的大题集合排序，计算小题数量
    	sortPaperDetailExps(paperDetailExps);
    	//给新的大题添加分数
    	scorePaperDetailExps(paperDetailExps);
    	//给新的大题下所有小题排序
    	sortPaperDetailUnitExps(paperDetailExps);
    	//给新的大题设置标题
    	setExpDtoTitle(paperDetailExps);
    	//如果每个小题分数不一样，题干后面添加分数
    	appendScoreToQuestionBody(paperDetailExps);
    	//给每个小题选项排序
    	setUnitExpNumber(paperDetailExps);
    	paperExpDto.setPaperDetails(paperDetailExps);
    	return paperExpDto;
    }

	/**
     * 将所有小题分类，添加到对应的大题下
     * @param paperDetails
     * @return
     */
    private List<PaperDetailExp> separatePaperDetailUntils(List<PaperDetail> paperDetails){
    	List<PaperDetailExp> paperDetailExps = new ArrayList<PaperDetailExp>();
    	//循环大题
    	for(PaperDetail paperDetail:paperDetails){
    		//获取小题
    		List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetail);
    		//set集合用来将小题分类
    		Set<Long> questionTypes = new HashSet<Long>();
    		//大题的临时集合，用来区分原卷每个大题
    		List<PaperDetailExp> subPaperDetailExps = new ArrayList<PaperDetailExp>();
    		//第一次循环小题，分成大题类型
    		for(PaperDetailUnit paperDetailUnit:paperDetailUnits){
    			//如果是套题
    			if(paperDetailUnit.getQuestionType().getName().equals("套题")){
    				isSubQuestion(paperDetailUnit,questionTypes);
    			}else {
    				Long questionTypeId = paperDetailUnit.getQuestionType().getId();
    				questionTypes.add(questionTypeId);
				}
    		}
    		//遍历Set，生成大题集合
    		for(Long questionType:questionTypes){
    			//生成大题对象
    			PaperDetailExp paperDetailExp = new PaperDetailExp();
    			paperDetailExp.setSortNumber(questionType);
    			subPaperDetailExps.add(paperDetailExp);
    		}
    		//第二次循环小题，添加到对应的大题下
    		for(PaperDetailUnit paperDetailUnit:paperDetailUnits){
    			//判断是否为套题
    			if(paperDetailUnit.getQuestionType().getName().equals("套题")){
    				addSubQuestion(paperDetailUnit,subPaperDetailExps);
    			}else {
    				PaperDetailUnitExp paperDetailUnitExp = BeanCopierUtil.copyProperties(paperDetailUnit, PaperDetailUnitExp.class);
        			//循环大题
        			for(PaperDetailExp paperDetailExp:subPaperDetailExps){
        				if(paperDetailExp.getSortNumber().longValue() == paperDetailUnitExp.getQuestionType().getId().longValue()){
        					List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
        					//如果大题下面没有小题，就生成新对象
        					if(paperDetailUnitExps == null){
        						paperDetailUnitExps = new ArrayList<PaperDetailUnitExp>();
        					}
        					paperDetailUnitExps.add(paperDetailUnitExp);
        					//将小题添加到对应大题下面
        					paperDetailExp.setPaperDetailUnits(paperDetailUnitExps);
        				}
        			}
				}
    		}
    		paperDetailExps.addAll(subPaperDetailExps);
    	}
    	return paperDetailExps;
    }
  
    /**
     * 大题序号重排，计算小题数量
     * @param paperDetailExps
     */
	private void sortPaperDetailExps(List<PaperDetailExp> paperDetailExps) {
		Collections.sort(paperDetailExps, new Comparator<PaperDetailExp>() {
			@Override
			public int compare(PaperDetailExp o1, PaperDetailExp o2) {
				if(o1.getSortNumber()>o2.getSortNumber()){
					return 1;
				}
				if(o1.getSortNumber()==o2.getSortNumber()){
					return 0;
				}
				return -1;
			}
		});
		if(paperDetailExps!=null && paperDetailExps.size()>0){
			//循环所有大题，给大题排序
			for(int i=0;i<paperDetailExps.size();i++){
				Integer number = i+1;
				paperDetailExps.get(i).setNumber(number);
				paperDetailExps.get(i).setCnNum(CommonUtils.toCHNum(number));
				int subNum =0;
				//循环小题，判断是否存在套题
				for(PaperDetailUnitExp paperDetailUnitExp:paperDetailExps.get(i).getPaperDetailUnits()){
					//如果有套题
					if(paperDetailUnitExp.getQuestionType().getName().equals("套题")){
						subNum = subNum + paperDetailUnitExp.getQuestion().getSubQuestions().size();
					}else {
						subNum = subNum + 1;
					}
				}
				paperDetailExps.get(i).setUnitCount(subNum);
			}
		}
	}

	/**
     * 每个大题添加分数
     */
    private void scorePaperDetailExps(List<PaperDetailExp> paperDetailExps) {
		for(PaperDetailExp paperDetailExp:paperDetailExps){
			List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
			Double totalScore = 0.0;
			//所有小题分数相加，得到大题分数
			for(PaperDetailUnitExp paperDetailUnitExp:paperDetailUnitExps){
				if(paperDetailUnitExp.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
					List<Question> subQuestions = paperDetailUnitExp.getQuestion().getSubQuestions();
					for(Question question:subQuestions){
						totalScore += question.getScore();
					}
				}else{
					totalScore += paperDetailUnitExp.getScore();
				}
			}
			paperDetailExp.setScore(totalScore);
		}
	}
    
    /**
     * 小题排序
     * @describle 大题排序完成之后，放在一个集合里面。然后对相同类型的大题分组，生成不同的结合，在对小题进行
     * @param paperDetailExps
     */
    private void sortPaperDetailUnitExps(List<PaperDetailExp> paperDetailExps) {
    	Map<Long,List<PaperDetailUnitExp>> map = new HashMap<Long, List<PaperDetailUnitExp>>();
    	//循环所有大题,将同类型的小题放到同一个集合下面
    	for(PaperDetailExp paperDetailExp:paperDetailExps){
    		List<PaperDetailUnitExp> paperDetailUnitExps = null;
    		//通过map的key去取value，如果没有，就添加
    		if(map.get(paperDetailExp.getSortNumber()) == null){
    			paperDetailUnitExps = new ArrayList<PaperDetailUnitExp>();
    		}else {
				paperDetailUnitExps = map.get(paperDetailExp.getSortNumber());
			}
    		paperDetailUnitExps.addAll(paperDetailExp.getPaperDetailUnits());
    		map.put(paperDetailExp.getSortNumber(), paperDetailUnitExps);
    	}
    	//循环所有小题，给小题排序
    	for(Long sortNumber:map.keySet()){
    		List<PaperDetailUnitExp> paperDetailUnitExps = map.get(sortNumber);
    		int number = 1;
    		if(paperDetailUnitExps != null && paperDetailUnitExps.size()>0){
    			for(int i = 0;i<paperDetailUnitExps.size();i++){
    				PaperDetailUnitExp paperDetailUnitExp = paperDetailUnitExps.get(i);
    				if(paperDetailUnitExp.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
    					List<Question> subQuestions = paperDetailUnitExp.getQuestion().getSubQuestions();
    					for(int j = 0;j<subQuestions.size();j++){
    						subQuestions.get(j).setNumber(number+j);
    					}
    					number += subQuestions.size();
    				}else {
    					paperDetailUnitExp.setNumber(number);
    					number++;
					}
        		}
    		}
    	}
	}

    /**
	 * 设置大题标题
	 * @param paperDetailExps
	 */
	public void setExpDtoTitle(List<PaperDetailExp> paperDetailExps) {
		for(PaperDetailExp paperDetailExp:paperDetailExps){
			String title = "";
			String titleDetail = null;
			String totalScore = BigDecimal.valueOf(paperDetailExp.getScore()).stripTrailingZeros().toPlainString();
			QuesStructType type = paperDetailExp.getPaperDetailUnits().get(0).getQuestionType();
			String questionScore = checkPaperDetailUnitScore(paperDetailExp);
			String scoreString = questionScore==null?"":"每小题"+questionScore+"分，";
			if (type == QuesStructType.SINGLE_ANSWER_QUESTION) {
				paperDetailExp.setName("单项选择题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore + "分）";
				titleDetail = "在每小题列出的备选项中只有一个符号合题目要求的，请将其选出并将“答题卡”的相应代码涂黑，错涂、多涂或未涂均无分";
			} else if (type == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
				paperDetailExp.setName("多项选择题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore+ "分）";
				titleDetail = "在每小题列出的备选项中有一个或多个符号合题目要求的，请将符合项选出并将“答题卡”的相应代码涂黑";
			} else if (type == QuesStructType.BOOL_ANSWER_QUESTION) {
				paperDetailExp.setName("判断题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore+ "分。"
						+ "正确的填涂√、错误填涂×。错涂、多涂或未涂均无分）";
			} else {
				paperDetailExp.setName(type.getName());
				title = "（本大题共"+paperDetailExp.getUnitCount()+"小题，"+scoreString+"共"+totalScore+"分）";
			}
			paperDetailExp.setTitle(title);
			paperDetailExp.setTitleDetail(titleDetail);
		}
	}
	
 	/**
	 * 校验大题下的小题分数是不是一样
	 * @param paperDetailExp
	 * @return  如果分数一样，返回分数
	 * 			如果分数不一样，返回 null
	 */
	public String checkPaperDetailUnitScore(PaperDetailExp paperDetailExp) {
		List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
		Set<Double> scoreSet = new HashSet<Double>();
		for (PaperDetailUnitExp unitExp : paperDetailUnitExps) {
			//判断套路
			if(unitExp.getQuestionType()==QuesStructType.NESTED_ANSWER_QUESTION){
				List<Question> subQuestions = unitExp.getQuestion().getSubQuestions();
				for(Question subQuestion:subQuestions){
					scoreSet.add(subQuestion.getScore());
				}
				/*List<Double> subScoreList =  unitExp.getSubScoreList();
				for(Double score:subScoreList){
					scoreSet.add(score);
				}*/
			}else{
				scoreSet.add(unitExp.getScore());
			}
		}
		if(scoreSet.size() == 1){
			return scoreSet.iterator().next()+"";
		}
		return null;
	}

	/**
     * 当大题下小题分数不是全部一样时，在小题题干后面加上小题的分数
     * @param details
     * @throws Exception
     */
    public void appendScoreToQuestionBody(List<PaperDetailExp> details) throws Exception{
    	for(PaperDetailExp paperDetailExp:details){
    		if(checkPaperDetailUnitScore(paperDetailExp)==null){
    			for(PaperDetailUnitExp paperDetailUnit:paperDetailExp.getPaperDetailUnits()){
    				//判断小题是否为套题
    				if(paperDetailUnit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION){
    					List<Question> subQuestions = paperDetailUnit.getQuestion().getSubQuestions();
    					for(Question subQuestion:subQuestions){
    						String questionBodyWord = appendScoreToQuestionBody(subQuestion.getQuesBodyWord(),subQuestion.getScore());
    						subQuestion.setQuesBodyWord(questionBodyWord);
    					}
    				}else {
    					Question question = paperDetailUnit.getQuestion();
        				String questionBodyWord = appendScoreToQuestionBody(question.getQuesBodyWord(),paperDetailUnit.getScore());
        				question.setQuesBodyWord(questionBodyWord);
					}
    			}
    		}
    	}
    }
    
    /**
     * 在题干上设置分数
     * @throws Exception 
     */
    public String appendScoreToQuestionBody(String quesBodyWordMl,double score) throws Exception{
    	String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index < pList.size()-1){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue("("+score+"分)");
            run.getContent().add(text);
            pContent.add(run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }
    
    /**
     * 套题添加到对应的大题
     * @param paperDetailUnit
     * @param subPaperDetailExps
     */
    private void addSubQuestion(PaperDetailUnit paperDetailUnit, List<PaperDetailExp> subPaperDetailExps) {
    	PaperDetailUnitExp paperDetailUnitExp = BeanCopierUtil.copyProperties(paperDetailUnit, PaperDetailUnitExp.class);
    	//得到套题下面所有小题
    	List<Question> questions = paperDetailUnit.getQuestion().getSubQuestions();
    	Set<Long> subQuestionType = new HashSet<Long>();
    	for(Question subQuestion:questions){
			subQuestionType.add(subQuestion.getQuestionType().getId());
		}
    	//如果全是同样的类型
    	if(subQuestionType.size()>0 && subQuestionType.size()<2){
			//获取所有题目，添加到对应大题下面
    		for (Long l:subQuestionType) {
				//循环已有的大题，进行添加套题
    			for(PaperDetailExp paperDetailExp:subPaperDetailExps){
    				if(paperDetailExp.getSortNumber().longValue() == l){
    					List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
    					//如果大题下面没有小题，就生成新对象
    					if(paperDetailUnitExps == null){
    						paperDetailUnitExps = new ArrayList<PaperDetailUnitExp>();
    					}
    					paperDetailUnitExps.add(paperDetailUnitExp);
    					//将小题添加到对应大题下面
    					paperDetailExp.setPaperDetailUnits(paperDetailUnitExps);
    				}
    			}
			}
		}else {
			//循环大题
			for(PaperDetailExp paperDetailExp:subPaperDetailExps){
				if(paperDetailExp.getSortNumber().longValue() == paperDetailUnitExp.getQuestionType().getId().longValue()){
					List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
					//如果大题下面没有小题，就生成新对象
					if(paperDetailUnitExps == null){
						paperDetailUnitExps = new ArrayList<PaperDetailUnitExp>();
					}
					paperDetailUnitExps.add(paperDetailUnitExp);
					//将小题添加到对应大题下面
					paperDetailExp.setPaperDetailUnits(paperDetailUnitExps);
				}
			}
		}
	}

	/**
     * 处理套题类型
     * @param paperDetailUnit
     */
    private void isSubQuestion(PaperDetailUnit paperDetailUnit,Set<Long> questionType){
    	//得到套题下面所有小题
		List<Question> questions = paperDetailUnit.getQuestion().getSubQuestions();
		Set<Long> subQuestionType = new HashSet<Long>();
		for(Question subQuestion:questions){
			subQuestionType.add(subQuestion.getQuestionType().getId());
		}
		//如果全是同样的类型
		if(subQuestionType.size()>0 && subQuestionType.size()<2){
			for(Long l:subQuestionType){
				questionType.add(l);
			}
		}else {
			Long questionTypeId = paperDetailUnit.getQuestionType().getId();
			questionType.add(questionTypeId);
		}
    }
	
	/**
	 * 给小题选项进行排序
	 * @param paperDetails
	 * @param startIndxt
	 * @throws Exception
	 */
	public void setUnitExpNumber(List<PaperDetailExp> paperDetails) throws Exception {
		for (PaperDetailExp paperDetail : paperDetails) {
			List<PaperDetailUnitExp> paperDetailUnitExpList = paperDetail.getPaperDetailUnits();
			for (PaperDetailUnitExp paperDetailUnit:paperDetailUnitExpList) {
				List<QuesOption> optionList = paperDetailUnit.getQuestion().getQuesOptions();
				if (optionList != null && optionList.size() > 0) {
					int index = 0;
					for (QuesOption quesOption : optionList) {
						quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),getOptionNum(index)));
						index++;
					}
				}
				List<Question> subQuesList = paperDetailUnit.getQuestion().getSubQuestions();
				Question question = paperDetailUnit.getQuestion();
				// 套题序号
				if (subQuesList != null && subQuesList.size() > 0) {
					question.setQuesBodyWord(replaceQuesBlank(question.getQuesBodyWord(),subQuesList.get(0).getNumber()));
					for (Question subQues : subQuesList) {
						subQues.setQuesBodyWord(setSubQuesNum(subQues.getQuesBodyWord(),subQues.getNumber()));
						subQues.setQuesAnswerWord(setSubQuesNum(subQues.getQuesAnswerWord(),subQues.getNumber()));
						subQues.setQuesBodyWord(replaceQuesBlank(subQues.getQuesBodyWord(),subQues.getNumber()));
						List<QuesOption> subOptionList = subQues.getQuesOptions();
						if (subOptionList != null && subOptionList.size() > 0) {
							int sub_index = 0;
							for (QuesOption quesOption : subOptionList) {
								quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),getOptionNum(sub_index)));
								sub_index++;
							}
						}
					}
				} else {
					question.setQuesBodyWord(setSubQuesNum(question.getQuesBodyWord(), paperDetailUnit.getNumber()));
					question.setQuesAnswerWord(setSubQuesNum(question.getQuesAnswerWord(),paperDetailUnit.getNumber()));
					question.setQuesBodyWord(replaceQuesBlank(question.getQuesBodyWord(),paperDetailUnit.getNumber()));
				}
			}
		}
	}
	
	/**
     * 设置选项号
     * @param optionWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setOptionNum(String optionWordMl,String num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + optionWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num+". ");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }
	
    /**
	 * 将数字1,2,3,4转化成A,B,C,D
	 * @param number
	 * @return
	 */
    public String getOptionNum(int number){
        char optionNum = (char)(65 + number);
        return String.valueOf(optionNum);
    }

    /**
     * 设置题号
     * @param quesBodyWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setSubQuesNum(String quesBodyWordMl,int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        for(Object pObj:pList){
            if(index > 0){
                break;
            }
            P p = (P) pObj;
            List<Object> pContent = p.getContent();
            R run = new R();
            Text text = new Text();
            text.setValue(num+". ");
            run.getContent().add(text);
            pContent.add(0,run);
            index++;
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }
	
    /**
     * 替换填空
     * 将###替换为下划线_______
     * @param wordMl
     * @param num
     * @return
     * @throws Exception
     */
	public String replaceQuesBlank(String wordMl,int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + wordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,Context.jc,Body.class);
        List<Object> pList = body.getContent();
        int index = 0;
        int cur = 0;
        Map<Integer,String> curMap = new HashMap<Integer,String>();
        for(Object pObj:pList){
            if(pObj.getClass().equals(P.class)){
                List<Object> pContent = ((P)pObj).getContent();
                for(Object rObj:pContent){
                    if(rObj.getClass().equals(R.class)){
                        List<Object> rContent = ((R)rObj).getContent();
                        for(Object tObj:rContent){
                            if (tObj instanceof JAXBElement)
                                tObj = ((JAXBElement<?>) tObj).getValue();
                            if(tObj.getClass().equals(Text.class)){
                                Text text = (Text)tObj;
                                String str = text.getValue();
                                //1
                                str = str.replaceAll("###","______");
                                text.setValue(str);
                                //2
                                Pattern pattern = Pattern.compile("##\\d{1,}##");
                        		Matcher m = pattern.matcher(str);
                            	while(m.find()){
                            		int curNum = num + index;
                                    String a = m.group();
                                    str = str.replaceAll(a, "___"+(curNum)+"___");
                                    text.setValue(str);
                                    index++;
                            	}
                            	//3
                                if(str.startsWith("#") || str.equals("___")){
                                    curMap.put(cur,str);
                                    text.setValue("");
                                }
                                //4
                                if(str.matches("^\\d{1,}$")){
                                    String preStr = curMap.get(cur - 1);
                                    if(!StringUtils.isEmpty(preStr) && preStr.startsWith("#")){
                                        int curNum = num + index;
                                        text.setValue("___"+(curNum)+"___");
                                        index++;
                                    }
                                }
                                cur++;
                            }
                        }
                    }
                }
            }
        }
        StringBuffer pWordMl = new StringBuffer();
        for(Object pObj:pList){
            if(pObj instanceof P){
                pWordMl.append(DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(pObj)));
            }
        }
        return pWordMl.toString();
    }

}
